--[[

RandomCompanion
Picks a random vanity pet or mount and automatically chooses the best type of mount for the job.

Usage:
Bind a key from the keybinding menu or...
/rmount or /rmt to choose a random mount
/rmount ground to choose a random ground mount in a flyable zone
/rpet or /rpt to choose a random vanity pet

TODO:
Config UI to disable mounts that you don't like or choose your favorite mounts more often...Coming very soon.
Add druid forms (aquatic form when swimming, travel form in combat, etc)
Re-summon your vanity pet after zoning or landing from a flight path

]]--

BINDING_HEADER_RANDOMCOMPANION = "RandomCompanion";
BINDING_NAME_RANDOMCOMPANION_MOUNT = "Random mount";
BINDING_NAME_RANDOMCOMPANION_MOUNT_GROUND = "Random ground mount";
BINDING_NAME_RANDOMCOMPANION_PET_SUMMON = "Random vanity pet";
BINDING_NAME_RANDOMCOMPANION_PET_DISMISS = "Dismiss vanity pet";

local DEBUG = nil;

RandomCompanion = {
	mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			bugs = {};
		};
	};
	critters = {
		weighted = {};
		unweighted = {};
	};
};

local function debugmsg(msg)
	if DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage('RandomCompanion Debug: ' .. msg);
	end
end

function RandomCompanion.Initialize(self, event, ...)
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion.mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			bugs = {};
		};
	};
	RandomCompanion.critters = {
		weighted = {};
		unweighted = {};
	};
	
	CreateFrame("GameTooltip", "RandomCompanionScanningTooltip", UIParent, "GameTooltipTemplate");
	RandomCompanionScanningTooltip:AddFontStrings(
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
	);
	
	for index = 1, GetNumCompanions("MOUNT") do
		local aq = false;
		local outland = false;
		local northrend = false;
		local very = false;
		local fast = false;
		local creatureid, name, spellid, icon, active = GetCompanionInfo("MOUNT", index);
		
		if name then
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				local text = _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
				
				if string.find(text, "Ahn'Qiraj") then
					aq = true;
				end
				if string.find(text, "Outland") then
					outland = true;
				end
				if string.find(text, "Northrend") then
					northrend = true;
				end
				if string.find(text, "very") then
					very = true;
				end
				if string.find(text, "fast") then
					fast = true;
				end
			end
			
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			
			local weight = RandomCompanion.GetWeight(name);
	
			if aq then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.bugs) + 1;
				RandomCompanion.mounts.unweighted.bugs[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif very and fast and outland and northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftflying) + 1;
				RandomCompanion.mounts.unweighted.swiftflying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif very and fast and not outland and not northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
				RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif not very and not fast and outland and northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.flying) + 1;
				RandomCompanion.mounts.unweighted.flying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif not very and not fast and not outland and not northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.ground) + 1;
				RandomCompanion.mounts.unweighted.ground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			else
				debugmsg(creatureName .. " is an unknown mount");
			end
			
			if weight > 0 then
				for counter = 1, weight do
					if aq then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.bugs) + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and outland and northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftflying) + 1;
						RandomCompanion.mounts.weighted.swiftflying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
						RandomCompanion.mounts.weighted.swiftground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and outland and northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.flying) + 1;
						RandomCompanion.mounts.weighted.flying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.ground) + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					else
						debugmsg(creatureName .. " is an unknown mount");
					end
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			debugmsg(index .. " index mount has no name?");
		end
	end
	
	RandomCompanionScanningTooltip:Hide()
	
	for index = 1, GetNumCompanions("CRITTER") do
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", index);
		if name then
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			local weight = RandomCompanion.GetWeight(name);
			
			local nextindex = table.getn(RandomCompanion.critters.unweighted) + 1;
			RandomCompanion.critters.unweighted[nextindex] = {
				creatureid = creatureid;
				name = name;
				index = index;
				spellid = spellid;
				icon = icon;
				active = active;
			}
			
			if weight > 0 then
				for counter = 1, weight do
					local nextindex = table.getn(RandomCompanion.critters.weighted) + 1;
					RandomCompanion.critters.weighted[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			debugmsg(index .. " index critter has no name?");
		end
	end
	
	local loadmessage = "";
	
	if event == "PLAYER_LOGIN" then
		if table.getn(RandomCompanion.mounts.unweighted.swiftflying) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftflying) .. " swift flyers";
		end
		if table.getn(RandomCompanion.mounts.unweighted.flying) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.flying) .. " slow flyers";
		end
		if table.getn(RandomCompanion.mounts.unweighted.swiftground) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftground) .. " swift grounders";
		end
		if table.getn(RandomCompanion.mounts.unweighted.ground) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.ground) .. " slow grounders";
		end
		if table.getn(RandomCompanion.mounts.unweighted.bugs) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.bugs) .. " bugs";
		end
		if table.getn(RandomCompanion.critters.unweighted) > 0 then
			if loadmessage ~= "" then
				loadmessage = loadmessage .. ", ";
			end
			loadmessage = loadmessage .. table.getn(RandomCompanion.critters.unweighted) .. " critters";
		end
		
		if loadmessage ~= "" then
			loadmessage = " with " .. loadmessage;
		end
		
		DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion loaded" .. loadmessage);
	end
end

function RandomCompanion.Mount(cmd)
	local flyable;
	
	if IsMounted() then
		Dismount();
	else
		if GetRealZoneText() == "Temple of Ahn'Qiraj" then
			if table.getn(RandomCompanion.mounts.weighted.bugs) > 0 then
				mountlist = "bugs";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif RandomCompanion.IsFlyableArea() and cmd ~= "ground" then
			if table.getn(RandomCompanion.mounts.weighted.swiftflying) > 0 then
				mountlist = "swiftflying";
			elseif table.getn(RandomCompanion.mounts.weighted.flying) > 0 then
				mountlist = "flying";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		else
			if table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		end
		
		if mountlist then
			local mountindex = random(table.getn(RandomCompanion.mounts.weighted[mountlist]));
			CallCompanion("MOUNT", RandomCompanion.mounts.weighted[mountlist][mountindex].index);
			debugmsg("Mounting " .. RandomCompanion.mounts.weighted[mountlist][mountindex].name);
		else
			if GetNumCompanions("MOUNT") > 0 then
				--We have mounts, but they aren't in the mount list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
				--Slight potential for an infinite loop here
				RandomCompanion.Initialize();
				RandomCompanion.Mount(cmd);
			else
				UIErrorsFrame:AddMessage("You have no mounts", 1.0, 0.0, 0.0, 53, 5);
			end
		end
	end
end

function RandomCompanion.IsFlyableArea()
	local northrend = {
		["Borean Tundra"] = "Borean Tundra";
		["Crystalsong Forest"] = "Crystalsong Forest";
		["Dalaran"] = "Dalaran";
		["Dragonblight"] = "Dragonblight";
		["Grizzly Hills"] = "Grizzly Hills";
		["Howling Fjord"] = "Howling Fjord";
		["Icecrown"] = "Icecrown";
		["Sholazar Basin"] = "Sholazar Basin";
		["Storm Peaks"] = "Storm Peaks";
		["Wintergrasp"] = "Wintergrasp";
		["Zul'Drak"] = "Zul'Drak";
	}
	if northrend[GetRealZoneText()] then
		isUsable, outOfMana = IsUsableSpell("Cold Weather Flying");
		if GetRealZoneText() == "Dalaran" then
			if GetMinimapZoneText() == "Krasus' Landing" then
				return true;
			else 
				if GetMinimapZoneText() == "The Underbelly" then 
					--Check coords to see if we're in the Sewer Pipe...Shamelessly stolen from Livestock
					local x, y = GetPlayerMapPosition("player")
					if x >= 0.092141166 and x <= 0.297127217055 and y >= 0.540120124816 and y <= 0.885304749012 then
						return true;
					else
						return false;
					end
				else
					return false;
				end
			end
		elseif GetRealZoneText() == "Wintergrasp" then
			return false;
		elseif IsFlyableArea() and isUsable then
			return true;
		else
			return false;
		end
	else
		return IsFlyableArea();
	end
end

function RandomCompanion.Pet()
	if table.getn(RandomCompanion.critters.weighted) > 0 then
		local petindex = random(table.getn(RandomCompanion.critters.weighted));
		RandomCompanion.PetDismiss();
		CallCompanion("CRITTER", RandomCompanion.critters.weighted[petindex].index);
		debugmsg("Summoning " .. RandomCompanion.critters.weighted[petindex].name);
	else
		if GetNumCompanions("CRITTER") > 0 then
			--We have pets, but they aren't in the pet list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
			--Slight potential for an infinite loop here
			RandomCompanion.Initialize();
			RandomCompanion.Pet();
		else
			UIErrorsFrame:AddMessage("You have no vanity pets", 1.0, 0.0, 0.0, 53, 5);
		end
	end
end

function RandomCompanion.PetDismiss()
	DismissCompanion("CRITTER");
end

function RandomCompanion.SetWeight(companion, value)
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] = value;
	debugmsg("RandomCompanion_settings["..GetRealmName().."]["..UnitName("player").."].weights["..companion.."] = "..RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion]);
end

function RandomCompanion.GetWeight(companion)
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] then
		return RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion];
	else
		return 1;
	end
end

function RandomCompanion.Config()
	RandomCompanionConfig:Show();
end

SlashCmdList["RANDOMMOUNT"] = RandomCompanion.Mount;
SLASH_RANDOMMOUNT1 = "/randommount";
SLASH_RANDOMMOUNT2 = "/rmount";
SLASH_RANDOMMOUNT3 = "/rmt";

SlashCmdList["RANDOMPET"] = RandomCompanion.Pet;
SLASH_RANDOMPET1 = "/randompet";
SLASH_RANDOMPET2 = "/rpet";
SLASH_RANDOMPET3 = "/rpt";

SlashCmdList["PETDISMISS"] = RandomCompanion.PetDismiss;
SLASH_PETDISMISS1 = "/petdismiss";
SLASH_PETDISMISS2 = "/pdismiss";

local frame = CreateFrame("Frame", "RandomCompanionFrame");
frame:SetScript("OnEvent", RandomCompanion.Initialize);
frame:RegisterEvent("COMPANION_LEARNED");
frame:RegisterEvent("PLAYER_LOGIN");
